"""

Part of the tagit module.
A copy of the license is provided with the project.
Author: Matthias Baumgartner, 2016

"""
# IMPORTS
import time

# INNER-MODULE IMPORTS
from ..controller import DataController

# EXPORTS
__all__ = ('CSidebox_Info_Cursor', )

## CODE ##
class CSidebox_Info_Cursor(DataController):
    """
    """
    def __init__(self, widget, model, settings, parent=None):
        super(CSidebox_Info_Cursor, self).__init__(widget, model, settings, parent)
        # Event bindings
        self.parent.bind(on_cursor=self.update)

    def __del__(self):
        self.parent.unbind(on_cursor=self.update)

    def update(self, browser, cursor):
        """Update the sidebox widget."""
        """When the cursor has changed."""
        if cursor is None: return False

        exp = self.model.attributes.exposure(cursor)
        exp = exp is None and '?' or round(exp, 2)

        info = [
              ('Date'        , time.strftime("%d:%m:%y, %H:%M", time.gmtime(self.model.get_creation_date(cursor))))
            , ('Size'        , "{} x {}".format(self.model.attributes.width(cursor) or '?', self.model.attributes.height(cursor) or '?'))
            , ('ISO'         , self.model.attributes.iso(cursor) or '?')
            , ('Exposure'    , '1 / {}'.format(exp))
            , ('Aperture'    , self.model.attributes.aperture(cursor) or '?')
            , ('Fx'          , self.model.attributes.focal_length_35(cursor) or '?')
            , ('Flash'       , self.model.attributes.flash(cursor) and 'Yes' or 'No')
            ]

        self.widget.update(info)


## EOF ##

