"""

Part of the tagit module.
A copy of the license is provided with the project.
Author: Matthias Baumgartner, 2016

"""
# IMPORTS

# INNER-MODULE IMPORTS
from ..controller import DataController

# EXPORTS
__all__ = ('CSidebox_Program_Control', )

## CODE ##
class CSidebox_Program_Control(DataController):
    """A set of functions to manage the model.
    """
    def __init__(self, widget, model, settings, parent=None, main_widget=None):
        super(CSidebox_Program_Control, self).__init__(widget, model, settings, parent)
        self.main_wx = main_widget
        self.pgm = None
        self.pgm_name = ''

    def status(self, message):
        self.widget.status = message

    def unload(self):
        if self.pgm is not None:
            if self.pgm.is_running():
                self.pgm.stop()

            self.pgm.unload()

    def select(self, program):
        self.unload()

        if program == 'BFS':
            from tagit.program import Pgm_BFS
            self.pgm = Pgm_BFS(self.status)
        elif program == 'DFS':
            from tagit.program import Pgm_DFS
            self.pgm = Pgm_DFS(self.status)

        self.pgm.init(self.main_wx, self.on_start, self.on_stop)
        self.pgm_name = program
        self.parent.dispatch('status', '[color=#FF0000][b]{}: [/b][/color] Program loaded'.format(self.pgm_name))

    def load(self, path):
        if path is not None:
            self.pgm.load(path)

    def save(self, path):
        self.pgm.save(path)
        self.parent.dispatch('status', '[color=#FF0000][b]{}: [/b][/color] Saved'.format(self.pgm_name))

    def on_stop(self):
        self.widget.show_play(self.widget.start)
        self.widget.show_load()
        self.parent.dispatch('status', '[color=#FF0000][b]{}: [/b][/color] Program stopped'.format(self.pgm_name))

    def on_start(self):
        self.widget.show_save()
        self.widget.show_pause()
        self.parent.dispatch('status', '[color=#FF0000][b]{}: [/b][/color] Started'.format(self.pgm_name))

    def start(self):
        if self.pgm is None: return
        self.pgm.start()

    def stop(self):
        if self.pgm is None or not self.pgm.is_running(): return
        self.pgm.stop()
        # continues with on_stop

    def pause(self):
        self.widget.show_play(self.widget.resume)
        self.pgm.pause()

    def resume(self):
        self.widget.show_pause()
        self.pgm.resume()

## EOF ##

