"""

Training requires:
    target -> candidates

    suggest new:
        at time t,
        * filter_t
        * model_t
        * target is next tag after t (query model)
        > model_t + filter_t -> set of images
        > model_t + filter_t -> set of candidates
            a shot in the foot if candidates are generated from trained data - or do recursive learning

    suggest rank:
        at time t
        * same as above
        * target is next tag in search after t (query log)

Part of the tagit module.
A copy of the license is provided with the project.
Author: Matthias Baumgartner, 2016
"""
# imports
from os.path import exists as pexists
from time import time
import sqlite3

# params
APPLICATION_ID = 683851074


## code ##

class DataLog(object):
    def __init__(self, path):
        # TODO: normalize path
        self._queue = []
        do_setup = not pexists(path)
        self.conn = sqlite3.connect(path)
        do_setup = do_setup or self.conn.execute('PRAGMA application_id').fetchone()[0] != APPLICATION_ID
        if do_setup: self.create()

    def __del__(self):
        self.empty_queue()

    def commit(self):
        try:
            self.conn.commit()
        except sqlite3.OperationalError, e:
            pass

    def empty_queue(self):
        try:
            # Final attempt to write queue
            for args in self._queue: self.write(*args)
            self.conn.commit()
            self._queue = []
        except sqlite3.OperationalError, e:
            pass

    def write(self, timestamp, sender, active, inactive):
        sender_id, sender_name = sender
        # FIXME: Also store candidates or search result (images) ?!
        # FIXME: Correlate candidate set with filter
        search_id = self.conn.execute("INSERT INTO search_history (t_logged, db_name, db_id) VALUES (?, ?, ?)", (timestamp, sender_name, sender_id)).lastrowid
        data = [(a, 1, search_id) for a in active] + [(a, 0, search_id) for a in inactive]
        self.conn.executemany("INSERT INTO search_token (token, active, search) VALUES (?, ?, ?)", data)

    def log_search(self, sender, active, inactive, timestamp=None):
        """Log a search query."""
        try:
            timestamp = timestamp is None and time() or timestamp
            for args in self._queue: self.write(*args) # Write queue first

            self.write(timestamp, sender, active, inactive) # Write message
            self.conn.commit()
            self._queue = []
        except sqlite3.OperationalError, e:
            # FIXME: SQL query misformulations lead to the same error type. Decide on message which should be 'database is locked'
            # FIXME: Distinguish WRITE lock (keep queue, attempt write again) and COMMIT lock (empty queue but re-commit)
            self._queue.append((timestamp, sender, active, inactive))

    def create(self):
        # TODO: Copy updater/schema versioning if it becomes necessary
        self.conn.executescript("""
            CREATE TABLE search_history (
                id INTEGER PRIMARY KEY,
                db_name TEXT,
                db_id TEXT,
                t_logged INTEGER unsigned DEFAULT 0
            );

            CREATE TABLE search_token (
                id INTEGER PRIMARY KEY,
                search INTEGER NOT NULL,
                active INTEGER default 1,
                token TEXT,
                FOREIGN KEY(search) REFERENCES search_history(id) ON DELETE CASCADE
            );
            """)

        self.conn.execute('PRAGMA application_id = {}'.format(APPLICATION_ID))
        self.conn.commit()

## EOF ##
