"""

Part of the tagit module.
A copy of the license is provided with the project.
Author: Matthias Baumgartner, 2016

"""
# STANDARD IMPORTS
import sqlite3

# INNER-MODULE IMPORTS
from updater import Updater, Updater_1
from updater_002 import Updater_2
from updater_003 import Updater_3
from updater_004 import Updater_4
from updater_005 import Updater_5
from tagit.basics import fst

# EXPORTS
__all__ = ('get_updater', 'get_version')

# CONSTANTS

_builder = {
      '1': Updater_1
    , '2': Updater_2
    , '3': Updater_3
    , '4': Updater_4
    , '5': Updater_5
    }


## CODE ##

def db_version(path):
    conn = sqlite3.connect(path)
    return fst(conn.execute('PRAGMA user_version').fetchone())

class Update_Chain(object):
    def __init__(self, path, verbose=False):
        self.conn = sqlite3.connect(path, check_same_thread=True)
        self.updaters = []
        self.verbose = verbose
    def append(self, inst):
        self.updaters.append(inst)
    def apply(self):
        for name, upd in self.updaters:
            if self.verbose:
                print "Update to version {0}".format(name)
            upd.apply(self.conn)

        self.conn.commit() # Otherwise, pragma is updated but not the rest...

def get_updater(path, v_src, v_trg, verbose):
    inst = Update_Chain(path, verbose)
    for v in range(v_src + 1, v_trg + 1):
        cls = _builder[str(v)]
        inst.append((str(v), cls()))

    return inst

## EOF ##
