"""Depth-first-search for Tagging.

Goes through the images in a depth-first manner. When a tag is
added to some selected images, they are shown exclusively.
Once no more tag is assigned to any of the visible images, the process
continues with the previous view, but excluding the last set.
This continues until the user stops the program.

This program enables the user to create hierarchies quickly.

Part of the tagit module.
A copy of the license is provided with the project.
Author: Matthias Baumgartner, 2016

"""
# IMPORTS

# INNER-MODULE IMPORTS
from tree_walker import Pgm_TreeWalker
from ..basics import subset
from ..bindings import Binding

# EXPORTS
__all__ = ('Pgm_DFS', )

## CODE ##

class Pgm_DFS(Pgm_TreeWalker):
    def on_add_tag(self, text, *args, **kwargs):
        #print "Tag added!", len(self.sets), len(self.sets[0])
        #print [sorted([self.base.index(im) for im in s]) for s in self.sets] # DEBUG

        # Allows empty tag on purpose as a way to skip

        sel = self._browser.selection
        if len(sel) > 0:
            vis = self.sets.pop(0)
            diff = [img for img in vis if img not in sel]
            if len(diff) > 0:
                self.sets.insert(0, diff) # Replace front with remaining images
            if len(text.strip()) > 0:
                self.sets.insert(0, sel) # Replace front with remaining images

        #print [sorted([self.base.index(im) for im in s]) for s in self.sets] # DEBUG

        # Actually add the tag
        r_value = self._add_tag(text, *args, **kwargs)

        # Continue
        if len(sel) > 0:
            if len(self.sets) == 0:
                self.stop()
            elif subset(self.sets[0], vis):
                # Shortcut: Show images exclusively
                self._browser.on_keyboard(self._browser.widget, ((Binding.ENTER, Binding.ENTER), ['ctrl']))
            else:
                # Temporarily reroute filter fu's
                apply_, self._filter.apply = self._filter.apply, lambda: False
                redraw_, self._filter.redraw = self._filter.redraw, lambda: True

                self._filter.go_back(len(self._filter.active)) # Remove all filters

                # Revert filter activity
                self._filter.apply = apply_
                self._filter.redraw = redraw_

                self._filter.selected_show(self.sets[0]) # Add exclusive token
                #self._filter.parent.dispatch('on_show_selected', self.sets[0]) # FIXME: Could use event

        self.notify()
        return r_value

## EOF ##

