"""Tree traversal for Tagging.

Part of the tagit module.
A copy of the license is provided with the project.
Author: Matthias Baumgartner, 2016

"""
# IMPORTS

# INNER-MODULE IMPORTS
from program import Program
from ..basics import subset
from ..bindings import Binding

# EXPORTS
__all__ = ('Pgm_TreeWalker', )

## CODE ##

class Pgm_TreeWalker(Program):
    def do_save(self, path):
        import json
        json.dump(self.sets, open(path, 'w'))

    def notify(self):
        n_sets = len(self.sets)
        n_itms = sum(map(len, self.sets))
        self._notifier("{} sets, {} items".format(n_sets, n_itms))

    def do_load(self, path):
        import json
        self.sets = json.load(open(path))
        self._filter.selected_show(self.sets[0]) # Add exclusive token
        self.notify()

    def decorate(self, wx):
        self._add_tag = wx.browser.controller.add_tag
        wx.browser.controller.add_tag = self.on_add_tag
        self._browser = wx.browser.controller
        self._filter = wx.filter.controller

    def undecorate(self, wx):
        wx.browser.controller.add_tag = self._add_tag

    def on_start(self):
        self.base_filter = self._filter.active[:], self._filter.inactive[:]
        #self.base = self._browser.images[:] # DEBUG
        self.sets = [self._browser.images[:]]
        if len(self.sets[0]) == 0:
            self.stop()

        #print [sorted([self.base.index(im) for im in s]) for s in self.sets] # DEBUG

    def on_stop(self):
        self._filter.active, self._filter.inactive = self.base_filter
        self._filter.redraw()
        self._filter.apply()

    def on_pause(self):
        self.filter_paused = self._filter.active

    def on_resume(self):
        self._filter.active = self.filter_paused
        self._filter.inactive = []
        self._filter.redraw()
        self._filter.apply()
        self.notify()

    def on_add_tag(self, text, *args, **kwargs):
        raise NotImplementedError()

## EOF ##
