"""

Part of the tagit module.
A copy of the license is provided with the project.
Author: Matthias Baumgartner, 2017
"""
# imports
from math import log

# inner-module imports
from feature import Feature
from tagit.basics import union

# exports
__all__ = ('ImageRelated', )


## code ##

log2 = lambda x: log(x) / log(2.0)

class ImageRelated(Feature):
    def split_ratio_all(self, tag): # Split ratio w.r.t all images
        # len(img with tag) / len(imgs)
        return 1.0 * len(self.graph.tag_by_name(tag)) / self.graph.n_images()

    def having(self, tag): # Number of images having tag
        return 1.0 * len(self.graph.tag_by_name(tag))

    def not_having(self, tag): # Number of images not having tag
        #len(img w/o tag) = len(img) - len(img with tag)
        return 1.0 * self.graph.n_images() - len(self.graph.tag_by_name(tag))

    def all_candidates(self, tag): # Number of images having all candidates
        # len(img with all candidates)
        all_ = list(self.graph.tag_by_name(tag))
        for cand in self.candidates: # Pairwise union, iteratively
            imgs = list(self.graph.tag_by_name(cand))
            all_ = union(all_, imgs)

        return 1.0 * len(all_)

    def info_all(self, tag): # Self-Information w.r.t all iamges
        # -log( imgs with tag / len(imgs) )
        info = 1.0 * len(self.graph.tag_by_name(tag)) / self.graph.n_images()
        if info == 0.0: return 0.0 # FIXME
        return -1.0 * log2(info)

    def freq_tags(self, tag): # Frequencies: total Images having tags / # tags
        # imgs with tag / len(tags)
        return 1.0 * len(self.graph.tag_by_name(tag)) / self.graph.n_tags()

    def freq_images(self, tag): # Frequencies: total Images having tags / # images
        # imgs with tag / len(imgs)
        return 1.0 * len(self.graph.tag_by_name(tag)) / self.graph.n_images()

    #def no_other(self, tag): # Number of images having no other candidate
    #    len(img with tag) - len(img with other candidate)

    #def all_but_tag(self, tag): # Number of images having all candidates but tag
    #    [img for images if all(c in img for c in candidates) and not tag in img]

    #def ntags(self, tag) : # Aggregated number of tags assigned images having tag
    #def ncandidates(self, tag) : # Aggregated number of candidates assigned images having tag

    #* Aggregated # candidates / # tags for all images having tag

    #def foo(self, tag): # Cardinality of tags appearing in at least one image having tag
    #    len(tags) for tags in img if img has tag

## EOF ##
