"""

Sources of tags
 * WordNet
 * dbPedia
 * dictionary
 * internet search
 * current tags

TODO: Add hypernyms, hyponyms, synonyms

Part of the tagit module.
A copy of the license is provided with the project.
Author: Matthias Baumgartner, 2017

"""
# imports
from nltk.corpus import wordnet as wn
from nltk.corpus.reader.wordnet import Synset, Lemma

# inner-module imports
from tagit.basics import fst

# exports
__all__ = ('ALL_PRODUCERS', 'wordnet', 'present')

## code ##

parse_synset_name = lambda name: fst(name.split('.'))
parse_lemma_name  = lambda name: name.replace('_', ' ')

def present_any(model, images):
    return model.tags.get(images)

def defined(model, images):
    return model.tags.get_all()

def present_all(model, images):
    N = len(images)
    hist = model.tags.histogram(include=images)
    return [tag for tag, freq in hist.iteritems() if freq == N]

def wordnet(model, images):
    suggestions = []
    for tag in model.tags.get(images):
        if len(wn.synsets(tag)) > 0:
            syn = wn.synsets(tag)[0] # pick first match

            # Cannonical form
            suggestions.append(parse_synset_name(syn.name()))

            # Lemmas
            suggestions.extend(
                map(parse_lemma_name,
                map(Lemma.name,
                    syn.lemmas()
                )))

            # Hypernyms, 1st order
            #suggestions.extend(
            #    map(parse_synset_name,
            #    map(Synset.name,
            #        syn.closure(Synset.hypernyms, depth=1)
            #    )))

            # Hyponyms, 1st order
            #suggestions.extend(
            #    map(parse_synset_name,
            #    map(Synset.name,
            #        syn.closure(Synset.hyponyms, depth=1)
            #    )))

    return suggestions

ALL_PRODUCERS = (wordnet, present_all, present_any, defined)

## EOF ##
