"""Browser actions.

Part of the tagit module.
A copy of the license is provided with the project.
Author: Matthias Baumgartner, 2016
"""
# imports
from os.path import join, dirname
from kivy.lang import Builder
from kivy.uix.image import Image
from kivy.uix.behaviors import ButtonBehavior
from kivy.uix.label import Label
from kivy.properties import StringProperty

# inner-module imports
from tagit.external import Tooltip
from tagit.controller.browser import CBrowser
from proxy_mixin import proxy_mixin

# exports
__all__ = ('VBrowserAction_GoFirst', 'VBrowserAction_PreviousPage', 'VBrowserAction_ScrollUp', 'VBrowserAction_ScrollDown', 'VBrowserAction_NextPage', 'VBrowserAction_GoLast', 'VBrowserAction_RotateLeft', 'VBrowserAction_DeleteImage', 'VBrowserAction_RotateRight')

# Load kv
Builder.load_file(join(dirname(__file__), 'browser.kv'))

class VBrowserAction(proxy_mixin, ButtonBehavior, Image, Tooltip):
    RCPT = CBrowser
    tooltip = StringProperty('')

    def on_tooltip(self, callee, text):
        self.set_tooltip(text)

class VBrowserAction_GoFirst(VBrowserAction):
    PROXY = 'FirstImage'

class VBrowserAction_PreviousPage(VBrowserAction):
    PROXY = 'PreviousPage'

class VBrowserAction_ScrollUp(VBrowserAction):
    PROXY = 'ScrollUp'

class VBrowserAction_ScrollDown(VBrowserAction):
    PROXY = 'ScrollDown'

class VBrowserAction_NextPage(VBrowserAction):
    PROXY = 'NextPage'

class VBrowserAction_GoLast(VBrowserAction):
    PROXY = 'LastImage'

class VBrowserAction_RotateLeft(VBrowserAction):
    PROXY = 'RotateLeft'

class VBrowserAction_DeleteImage(VBrowserAction):
    PROXY = 'DeleteImage'

class VBrowserAction_RotateRight(VBrowserAction):
    PROXY = 'RotateRight'

## EOF ##
