"""

Part of the tagit module.
A copy of the license is provided with the project.
Author: Matthias Baumgartner, 2016
"""
# imports

# inner-module imports
from tagit.controller.actions.proxy import recursive_proxy

# exports
__all__ = ('proxy_mixin', )


## CODE ##

class proxy_mixin(object):
    """Bind to and call proxy.

    Calls go through the *apply* method. The proxy is retreived
    upon first call, setup or parent change.

    If no proxy is available, the call is silently ignored.

    The subclass is expected to set
        RCPT : Recipient of the request (class)
        PROXY: Proxy name (string)

    """
    def apply(self): # Is overwritten after the proxy is known.
        # get proxy
        p = recursive_proxy(self.get_controller(), self.RCPT, self.PROXY)
        # repace and call
        if p is not None:
            self.apply = p
            self.apply()

    def on_startup(self, *args, **kwargs):
        # get proxy
        p = recursive_proxy(self.get_controller(), self.RCPT, self.PROXY)
        # repace
        if p is not None:
            self.apply = p

    def on_parent(self, *args, **kwargs):
        # get proxy
        p = recursive_proxy(self.get_controller(), self.RCPT, self.PROXY)
        # repace
        if p is not None:
            self.apply = p

## EOF ##
