"""Dialogue which shows keyboard bindings.
"""
# imports
from kivy.properties import StringProperty
from os.path import join, dirname
from kivy.lang import Builder
from dialogue import Dialogue
from kivy.uix.label import Label

# exports
__all__ = ('BindingsDialogue', )

# Load kv
Builder.load_file(join(dirname(__file__), 'dlg_bindings.kv'))

# Classes
class BindingsDialogue_Label(Label): pass

class BindingsDialogue(Dialogue):
    """Dialogue with some text.

    Set the default text as **text**.

    """
    def __init__(self, bindings, *args, **kwargs):
        self._keyboard_ctrl = kwargs.pop('keyboard_ctrl', None)
        super(BindingsDialogue, self).__init__(*args, **kwargs)

        if self._keyboard_ctrl is not None:
            self._keyboard_ctrl.request_exclusive_keyboard()


        add_left = lambda txt, **kwargs: self.left_col.add_widget(BindingsDialogue_Label(text=txt, **kwargs))
        add_right = lambda txt, **kwargs: self.right_col.add_widget(BindingsDialogue_Label(text=txt, **kwargs))

        for title, binds in bindings:
            add = title.lower() == 'browser' and add_right or add_left # FIXME: Ugly, but hey, it works

            # Section
            add('[b][i][color=#0000FF]{0}[/color][/i][/b]'.format(title), padding=(0, 20))
            add('')

            # Bindings
            for c, e in binds:
                add(c)
                add(e)

    def dismiss(self):
        if self._keyboard_ctrl is not None: # release keyboard
            self._keyboard_ctrl.release_exclusive_keyboard()
        super(BindingsDialogue, self).dismiss()

## EOF ##
