"""

Part of the tagit module.
A copy of the license is provided with the project.
Author: Matthias Baumgartner, 2016

"""
# imports
from os.path import join, dirname
from kivy.lang import Builder
from kivy.uix.boxlayout import BoxLayout
from kivy.uix.button import Button
from kivy.uix.boxlayout import BoxLayout
from kivy.gesture import Gesture, GestureDatabase
from kivy.uix.behaviors import DragBehavior

# inner-module imports
from ..controller.suggestion import CSuggestion
from gestures import SWIPE_RIGHT, SWIPE_LEFT

# exports
__all__ = ('VSuggestion', )

# Load kv
Builder.load_file(join(dirname(__file__), 'suggestion.kv'))

class VSuggButton(DragBehavior, Button):
    def __init__(self, *args, **kwargs):
        self.on_swipe_left = kwargs.pop('on_swipe_left', lambda x: None) # TODO: Use events maybe
        self.on_swipe_right = kwargs.pop('on_swipe_right', lambda x: None) # TODO: Use events maybe
        super(VSuggButton, self).__init__(*args, **kwargs)
        self.gdb = GestureDatabase()
        self.gdb.add_gesture(SWIPE_RIGHT)
        self.gdb.add_gesture(SWIPE_LEFT)

    def on_touch_down(self, touch):
        if self.collide_point(*touch.pos):
            touch.ud['origin']  = self
            touch.ud['origpos'] = (self.x, self.y)
            touch.ud['gesture'] = [(touch.x, touch.y)]

        return super(VSuggButton, self).on_touch_down(touch)

    def on_touch_move(self, touch):
        if 'origin' in touch.ud and touch.ud['origin'] == self:
            try:
                # store points of the touch movement
                touch.ud['gesture'].append((touch.x, touch.y))
            except (KeyError) as e:
                pass

        #touch.dy, touch.dx = 0, max(0, touch.dx) # For dragging: Allow right move only
        touch.dy = 0
        return super(VSuggButton, self).on_touch_move(touch)

    def on_touch_up(self, touch):
        if 'origin' in touch.ud and touch.ud['origin'] == self:
            g = Gesture()
            g.add_stroke(touch.ud['gesture'])
            g.normalize()
            det = self.gdb.find(g, minscore=0.70)
            if det is not None:
                if det[1] == SWIPE_RIGHT:
                    self.on_swipe_right(self)
                elif det[1] == SWIPE_LEFT:
                    self.on_swipe_left(self)
            else:
                # Restore position to point before dragging
                self.x, self.y = touch.ud['origpos']

        return super(VSuggButton, self).on_touch_up(touch)


class VEditButton(BoxLayout): pass

class VSuggestion(BoxLayout):
    def __init__(self, *args, **kwargs):
        super(VSuggestion, self).__init__(*args, **kwargs)
        self.gdb = GestureDatabase()
        self.gdb.add_gesture(SWIPE_RIGHT)
        self.gdb.add_gesture(SWIPE_LEFT)

    def on_parent(self, *args):
        """Set the controller."""
        self.controller = self.get_controller(CSuggestion)
        super(VSuggestion, self).on_parent(*args)

    def redraw(self, suggestions):
        self.tagspace.clear_widgets()
        self.suggestions = suggestions
        for s in self.suggestions[:10]:
            self.tagspace.add_widget(VSuggButton(text=s, on_release=self.add, on_swipe_right=self.blacklist, on_swipe_left=self.add))

    def blacklist(self, btn):
        self.controller.add_blacklist(btn.text)
        if btn.text in self.suggestions: self.suggestions.remove(btn.text)
        self.redraw(self.suggestions)

    def add(self, btn):
        self.controller.add_item(btn.text)

    def switch_mode(self, btn):
        if btn.state == 'down':
            self.controller.set_mode_filter()
        elif btn.state == 'normal':
            self.controller.set_mode_suggest()

    def on_touch_down(self, touch):
        if not self.collide_point(*touch.pos):
            touch.ud['gesture'] = [(touch.x, touch.y)]
            touch.ud['origin']  = self
        return super(VSuggestion, self).on_touch_down(touch)

    def on_touch_move(self, touch):
        if 'origin' in touch.ud and touch.ud['origin'] == self:
            try:
                # store points of the touch movement
                touch.ud['gesture'].append((touch.x, touch.y))
            except (KeyError) as e:
                pass

        return super(VSuggestion, self).on_touch_move(touch)

    def on_touch_up(self, touch):
        if 'origin' in touch.ud and touch.ud['origin'] == self:
            g = Gesture()
            g.add_stroke(touch.ud['gesture'])
            g.normalize()
            det = self.gdb.find(g, minscore=0.70)
            if det is not None and det[1] == SWIPE_RIGHT:
                self.controller.go_back(1)
                pass

        return super(VSuggestion, self).on_touch_up(touch)

## EOF ##
